function [wsx, wpx] = CTUSWinN(N, beta, alpha)
% CTUSWinN  Sampled continuous-time ultraspherical window
% w = CTUSWinN(N, beta, alpha)
% w     <- Window coefficients (column vector)
% N      -> Number of samples
% beta   -> Window parameter (trades off main lobe width against sidelobe level)
% alpha  -> Window type, alpha >= 0
%
% Sample a continuous-time ultraspherical window with -T/2 <= t <= T/2. The
% sample points are
%   t[n] = (n - (N-1)/2) dt, 0 <= n <= N-1.
% For alpha > 0
%   dt = T/N     => t[0] = -T/2 (N-1)/N; t[N-1] = +T/2 (N-1)/N
%   This is a modified sampling pattern.
% For alpha = 0
%   dt = T/(N-1) => t[0] = -T/2; t[N-1] = +T/2
%   This is a conventional sampling pattern.
%
% A truncated series approximation to the continuous-time window function is
% employed.
%
% For alpha > 0, the CT window is a smooth function. For alpha = 0, the CT
% window is the sum of a smooth function and a function consisting of delta
% functions at either edge of the window. In the sampled window, the delta
% functions are replaced by adding (N-1)/2 to the ends of the window.
%
% This function can also be called with two output vectors,
% [wsx, wpx] = CTUSWin(N, beta, alpha);
% This calling sequence is useful for alpha = 0.
% wsx  <-  Window coefficients corresponding to samples of the smooth part of a
%          continuous-time ultraspherical window (column vector)
% wpx  <-  Window coefficients corresponding to delta functions in the
%          continuous-time ultraspherical window (column vector). This is an all
%          zero vector for alpha > 0. For alpha = 0, the first and last
%          elements are non-zero. The overall window is the sum of ws and wp.

% For most combinations of beta and alpha, the window is normalized by the value
% of the continuous-time window at t = 0. For alpha = 0 and beta = 0, the smooth
% part of the CT window is all zeros (ws is all zeros) and wp has two non-zero
% values equal to (N-1)/2 at the ends of the window.

% $Id: CTUSWinN.m,v 1.8 2025/01/29 15:53:13 pkabal Exp $

% Initialize the window vector
wsx = NaN(N, 1);
wpx = zeros(N, 1);
if (mod(N, 2) == 0)
  Nh = N/2 - 1;
else
  Nh = (N-1)/2;
end

% Sampled continuous-time Dolph-Chebyshev window
if (alpha == 0)
  if (N > 1)
    dtau = 2 / (N-1);     % Conventional sampling pattern
  else
    dtau = 1;
  end
  if (beta == 0)
    wNorm = 1;
  else
    wNorm = (beta^2/2) * BesselSum(beta^2, 1);
  end
  for n = 0:Nh
    tau = (n - (N-1)/2) * dtau;
    tau2 = 1 - tau^2;
    wsx(n+1) = ((beta^2)/2) / wNorm * BesselSum(beta^2*tau2, 1);
  end
  wpx(0+1) = ((N-1)/2) / wNorm;
  wpx(end) = wpx(0+1);

% Sampled continuous-time Kaiser-Bessel window
elseif (alpha == 1)
  dtau = 2 / N;           % Modified sampling pattern
  wNorm = BesselSum(beta^2, 0);
  for n = 0:Nh
    tau = (n - (N-1)/2) * dtau;
    tau2 = 1 - tau^2;
    wsx(n+1) = BesselSum(beta^2*tau2, 0) / wNorm;
  end

else
  dtau = 2 / N;           % Modified sampling
  nu = alpha - 1;
  wNorm = BesselSum(beta^2, nu);
  for n = 0:Nh
    tau = (n - (N-1)/2) * dtau;
    tau2 = 1 - tau^2;
    wsx(n+1) = tau2^nu / wNorm * BesselSum(beta^2*tau2, nu);
  end
end

% Fill in the symmetrical part
for n = 0:Nh
  wsx(N-1-n+1) = wsx(n+1);
end

if (nargout() == 1 && alpha == 0)
  wsx = wsx + wpx;
end

end

% ----- -----
function S = BesselSum(z2, nu)

S = 1;
tol = 1e-8;
conv = false;
sk = 1;
for k = 1:25
  sk = (z2 / (4*k*(nu+k))) * sk;
  S = S + sk;
  if (sk < tol*S)
    conv = true;
    break
  end
end
if (~conv)
  warning('BesselSum - Tolerance not reached');
end

end
