function W = FRCTUSWin(f, T, beta, alpha)
% FRCTUSWin  Frequency response of a continuous-time ultraspherical window
% W - FRCTUSWin(f, T, beta, alpha)
% W    <-  Frequency response sampled at times given by f
% f     -> Evaluation frequencies (Hz)
% T     -> Non-zero extent of the window is -T/2 <= t <= T/2
% beta  -> Window parameter (trades off main lobe width against sidelobe level)
% alpha -> Window type, alpha >= 0

% R. L. Streit, "A Two-Parameter Family of Weights for Nonrecursive Digital
%   Filters and Antennas", IEEE trans. Acoustics, Speech, Signal Proc.,
%   vol. ASSP-32, No. 1, pp. 108-118, Feb. 1984.

% $Id: FRCTUSWin.m,v 1.4 2024/10/31 19:04:36 pkabal Exp $

% Change of nomenclature to match Streit
%   L           -> T/2    - window length T
%   tau         -> beta   - as in Kaiser-Bessel windows

L = T/2;
v = 2*pi * f;

W = FRCTStreitWin(L*v, beta, alpha);

end

% ----- -----

% In Streit, H(v) is given in Eq. (58)
%   H(v) = D(mu,tau) J_(mu-1/2)[L s(v,tau/L)] / (L s(v,tau/L)^(mu-1/2))
%        = D(mu,tau) J_(mu-1/2)[s(Lv, tau)] / s(Lv, tau)^(mu-1/2)
% where s(a,b) = sqrt(a^2-b^2).
%
% Let H(v) be normalized so that H(0) = 1. Then
%   H(0) = D(mu,tau) J_(mu-1/2)[i tau] / (i tau)^(mu-1/2)
%        = 1
% This gives
%   D(mu,tau) = (i tau)^(mu-1/2) / J_(mu-1/2)[i tau].
% The bessel function J_a(i r) with an imaginary argument can be expressed as
% a modified bessel function,
%   J_a(i x) = i^a I_a(x).
% Then
%   D(mu,tau) = tau^(mu-1/2) / I_(mu-1/2)[tau].
% Then
%   H(v) = D(mu,tau) J_(mu-1/2)[s(vL,tau)] / s(Lv,tau)^(mu-1)}.
%
% Further,
%   s(v,tau) = i s(tau,v), v <= tau
%            = s(v,tau),   v >= tau
%
% Then using only real arithmetic
%   H(v) = D(mu,tau) I_(mu-1/2)(s(tau,Lv)) / s(tau,Lv)^(mu-1/2),  v <= tau
%        = D(mu,tau) J_(mu-1/2)(s(Lv,tau)) / s(Lv,tau)^(mu-1/2),  v >= tau

function H = FRCTStreitWin(w, beta, alpha)

mu = alpha-1/2;
s = @(a, b)(sqrt(a.^2 - b.^2));
HLx = @(w, alpha, beta) (besseli(mu, s(beta,w)) ./ (s(beta,w).^mu));
HHx = @(w, alpha, beta) (besselj(mu, s(w,beta)) ./ (s(w,beta).^mu));

D0 = 2^mu * gamma(mu+1);
if (beta == 0)
  D = D0;
else
  D = beta^mu / besseli(mu, beta);
end

I = (abs(w) < beta);
J = (abs(w) == beta);
K = (abs(w) > beta);
H(I) = D * HLx(w(I), alpha, beta);
H(J) = D / D0;
H(K) = D * HHx(w(K), alpha, beta);

end
