function sigma = FindUSWinMLW(N, x0, alpha)
% FindUSWinMLW  Find the width of the main lobe of a ultraspherical window.
% sigma = FindUSWinMLW(N, x0, alpha)
% sigma  <-  Main lobe width relative to the main lobe width of a rectangular
%            window.
% N       -> Number of window coefficients
% x0      -> Location of the ultraspherical polynomial in x corresponding to the
%            peak of the spectrum of the window
% alpha   -> Window parameter
%            alpha=0 gives the Dolph-Chebyshev window (based on a Chebyshev
%              polynomial of the first kind)
%            alpha=1 gives the Saramäki window (based on a Chebyshev polynomial
%              of the second kind)
%
% The main lobe of the spectrum of a window is defined by the zero crossings
% surrounding the main lobe (centred at frequency 0). The frequency separation
% of the zero points defines the main lobe width. For reference, in terms of
% normalized frequency (Hz), the main lobe width for a rectangular window is 2/N
% and the main lobe width for a Hann window is 4/N.
%
% This routine returns sigma which is the relative main lobe width of the
% ultraspherical window with respect to the main lobe width of a rectangular
% window. In normalized frequency the main lobe width of the ultraspherical
% windows is sigma*2/N. For the main lobe width in radians, multiply by 2*pi.

if (N == 1)
  sigma = NaN;
  return;
elseif (N == 2)
  sigma = N/2;
  return;
end

M = N - 1;

if (alpha == 0 || alpha == 1)
  x1 = EstZero(1, M, alpha);

else
  % Estimate the midway point between the first and second zeros
  x12 = 0.5 * (EstZero(1, M, alpha) + EstZero(2, M, alpha));

  % Search in the interval [x12, x0] for a zero crossing
  x1 = fzero(@USPoly, [x12, x0], optimset('TolX', 1e-14), M, alpha);
  if (isnan(x1))
    error('FindUSWinMLW: Failed to find the polynomial zero');
  end
end

% x1 = x0 cos(w1/2), solve for w1 from x1 and x0
sigma = (N/pi) * acos(x1/x0);  % Main lobe width expansion factor

end

% -----
function xr = EstZero(k, M, alpha)

% Estimate of the k'th zero of Pm(x)
% k = 1 for largest zero, k = 2 for the next largest zero, ...

% Interpolate/extrapolate the exact (inverse) roots of the Chebyshev polynomials
% (first and second kind)
xL0 = cos(pi * (k - 1/2) / M);    % Exact roots for alpha = 0
xL1 = cos(pi * k / (M+1));        % Exact roots for alpha = 1
xr = (xL0 * xL1) / (alpha * xL0 + (1 - alpha) * xL1);

end
