function RdB = FindUSWinRdB(N, x0, alpha)
% FindUSWinRdB  Find the ratio of the peak of an ultraspherical window to the
% amplitude of the first sidelobe
% RdB = FindUSWinRdB(N, x0, alpha)
% RdB  <-  Peak to sidelobe ratio in dB
% N     -> Number of window coefficients
% x0    -> Location of the ultraspherical polynomial in x corresponding to the
%          peak of the spectrum of the window
% alpha -> Window parameter 
%          alpha=0 gives the Dolph-Chebyshev window (based on a Chebyshev
%            polynomial of the first kind)
%          alpha=1 gives the Saramäki window (based on a Chebyshev polynomial
%            of the second kind)

% $Id: FindUSWinRdB.m,v 1.8 2025/03/19 14:46:29 pkabal Exp $

% Look for the first sidelobe in the interval just below x = 1. The search range
% is delimited by [xL, 1], where xL is an estimate of the zero location based on
% interpolating the zero locations of Chebyshev polynomials of the first and
% second kind.

dBV = @(x) (20*log10(x));

if (N <= 2)
  RdB = NaN;
  return;
end

% For alpha = 0, determine R by formula
if (alpha == 0)
  R = cosh((N-1)*acosh(x0));

else
  M = N - 1;
  xU = 1;
  xL = EstZero(2, M, alpha);

  % Find the sidelobe amplitude in the interval [xL, xU]
  [xMin, SL] = fminbnd(@USP, xL, xU, optimset('TolX', 1e-14), M, alpha);
  if (xMin == xL || xMin == xU || SL >= 0)
    error('FindUSWinRdB: Failed to find the sidelobe level');
  end

  % Find the amplitude of the point x0
  Vx0 = USP(x0, M, alpha);
  R = abs(Vx0/SL);
end

RdB = dBV(R);

end

% ----- -----
function xr = EstZero(k, M, alpha)

% Estimate of the k'th zero of Cm(x)
% k = 1 for largest zero, k = 2 for the next largest zero, ...

% Interpolate/extrapolate the exact (inverse) roots of the Chebyshev polynomials
% (first and second kind)
xL0 = cos(pi * (k - 1/2) / M);    % Exact roots for alpha = 0
xL1 = cos(pi * k / (M+1));        % Exact roots for alpha = 1
xr = (xL0 * xL1) / (alpha * xL0 + (1 - alpha) * xL1);

end

% ----- -----
function v = USP(x, M, alpha)
% Ultraspherical polynomial, flipped in sign for alpha < 0

if (alpha < 0)
  v = -USPoly(x, M, alpha);
else
  v = USPoly(x, M, alpha);
end

end
