function w = SCTDolphChebyWin(N, beta)
% Generate a sampled Dolph-Chebyshev window with almost equiripple sidelobes.
% w = SCTDolphChebyWin(N, beta)
% w   <-  Symmetric window (column vector) of length N
% N    -> Window length
% beta -> Parameter for a continuous-time Dolph-Chebyshev window
%
% Start by finding N samples of a continuous-time Dolph-Chebyshev window. This
% window consists of samples of a smooth function with additional pulses at the
% end of the window. The frequency domain sidelobes of this sampled CT window
% are approximately equiripple. This routine modifies the end samples of the
% window to give a frequency response with sidelobes which are much closer to
% being equiripple.

% $Id: SCTDolphChebyWin.m,v 1.4 2025/04/03 14:26:45 pkabal Exp $

% Find the samples of the normalized smooth part of a continuous-time
% Dolph-Chebyshev window
alpha = 0;
w = CTUSWinN(N, beta, alpha);

% Modify the end samples of w
if (beta ~= 0)
  w = ModCheby1Win(w, beta);
end

end

% ----- ----
function h = ModCheby1Win(h, beta)
% Modify the end samples of a sampled CT Dolph-Chebyshev window such that
% frequency response is closer to having equiripple sidelobes.
% h   <-  Symmetric column vector. This is the same as h, except that the first
%         and last elements are modified.
% h    -> Symmetric column vector of window coefficients
% beta -> Window parameter, must be non-zero
%
% For N odd, the end elements of h are modified so as to ensure that the
% sidelobe peak at the half sampling frequency is of the correct value.
%
% For N even, the end elements of h are modified so as to set the slope of
% the sidelobe at the half sampling frequency.

R = cosh(beta);

N = length(h);
h(0+1) = 0;
h(N-1+1) = 0;

Bs0 = sum(h);

if (mod(N, 2) ~= 0)
  nv = (-1).^(0:N-1)';
  Bspi = sum(nv .* h);
  Bp0 = (Bs0 - Bspi)/(R-1) - Bspi;

else
  n = (0:N/2-1)';
  an = (-1).^n .* (1 - 2*n/(N-1));
  Fspi = 2 * sum(an .* h(n+1));

  x0 = cosh(beta/(N-1));
  Bp0 = (Bs0 - Fspi)/(R/x0-1) - Fspi;

end

h(0+1) = Bp0/2;
h(N-1+1) = Bp0/2;

end
