function y = USPoly(x, M, alpha, RedPoly)
% USPoly  Evaluate an Mth order ultraspherical polynomial at x.
% y = USPoly(x, M, alpha, RedPoly)
% y = USPoly(x, M, alpha)
% y    <-  Samples of the polynomial
% x       -> Vector of evaluation points
% M       -> Polynomial order for a polynomial with M+1 coefficients
% alpha   -> Ultraspherical parameter. For alpha=0, the ultraspherical
%            polynomial is scaled by 2/M to match the scaling of the Chebyshev
%            polynomial of the first kind.
% RedPoly -> Flag (true/false), optional, defaults to false
%            false - Samples y are from a polynomial of order M
%            true  - When M is odd, Samples y are from a polynomial of order M-1
%                    which is obtained by factoring out the root at x=0 from the
%                    polynomial of order M

% $Id: USPoly.m,v 1.15 2025/04/04 19:41:48 pkabal Exp $

if (alpha < 0 && fix(alpha) == alpha)
  error('USPoly: alpha cannot be a negative integer');
end

if (nargin() == 3)
  RedPoly = false;
end

yp = 1;         % m = 0
if (alpha == 0)
  y = 1;
  nu2 = 0;
else
  nu2 = 2*(alpha-1);
  y = 2*alpha;  % m = 1, y scaled by 1/x
end

if (M == 0)
  y = yp;
end

x2 = x.^2;
meven = false;
for m = 2:M
  meven = ~meven;
  ypp = yp;
  yp = y;

  Am = 2 + nu2/m;
  Bm = 1 + nu2/m;
  if (meven)
    y = Am*x2.*yp - Bm*ypp;  % m even, actual value
  else
    y = Am*yp - Bm*ypp;      % m odd, scaled by 1/x
  end
end

% Fix y for M odd if RedPoly is false
if (~meven && ~RedPoly)
  y = x .* y;
end

if (alpha == 0 && M > 0)
  y = 2*y / M;
end

end
