function h = USWin(N, x0, alpha, Norm)
% USWin  Window design using an ultraspherical polynomial.
% h = USWin(N, x0, alpha, Norm)
% h    <-  Symmetrical window coefficients (column vector), length N
% N     -> Number of window coefficients.
% x0    -> Location of the ultraspherical polynomial in x corresponding to the
%          peak of the spectrum of the window.
% N     -> Number of coefficients for the window
% alpha -> Window parameter, special cases
%          alpha=0 gives the Dolph-Chebyshev window (based on a Chebyshev
%            polynomial of the first kind)
%          alpha=1 gives the Saramäki window (based on a Chebyshev polynomial
%            of the second kind)
% Norm  -> Normalize flag (defaults to true if missing). If Norm is true, the
%          window coefficients are normalized by dividing the coefficients by
%          the element which has the largest absolute value - that element
%          becomes +1.
% This routine calculates the window as the inverse DFT of the sampled frequency
% response.

% $Id: USWin.m,v 1.11 2025/03/19 14:47:54 pkabal Exp $

% Calculation of the frequency response of the window:
% - Sample the polynomial at x = x0 cos(2 pi k/N). Then x is in the interval
%   [-x0, x0].
% - The window is found as the inverse DFT of the N samples.

if (nargin() < 4)
  Norm = true;
end

if (mod(N, 2) ~= 0)
  Ne = (N-1)/2;
else
  Ne = N/2-1;
end

% Sample the polynomial
fk = (0:Ne) / N;     % (N+1)/2 values (N odd) or N/2 values (N even)
wk = 2*pi*fk;

% Sample the frequency response
x = x0 * cos(wk/2);
B = USPoly(x, N-1, alpha);

% N odd   B[0] B[1] B[2] {B[2] B[1]}       0:(N-1)/2
% N even  B[0] B[1] B[2] {0 -B[2] -B[1]}   0:N/2-1 

h = NaN(N, 1);
nm = (N-1)/2;   % Shift (half integer for N even)

if (mod(N, 2) ~= 0)
  k = 1:Ne;         % 1:(N-1)/2
  for n = 0:Ne-1    % 0:(N-1)/2-1
    h(n+1) = B(0+1) + 2 * B(k+1) * cos(2*pi*(n-nm)*k/N)';
    h(N-1-n+1) = h(n+1);
  end
  h(nm+1) = B(0+1) + 2 * sum(B(k+1));
else
  k = 1:Ne;         % 1:(N/2-1)
  for n = 0:Ne      % 0:(N/2-1)
    h(n+1) = B(0+1) + 2 * B(k+1) * cos(2*pi*(n-nm)*k/N)';
    h(N-1-n+1) = h(n+1);
  end
end

% Normalize and make the largest value positive
if (Norm)
  [~, k] = max(abs(h));
  h = h / h(k);
else
  h = h / N;   % Inverse DFT scaling
end

end
