function x0 = USWinMLW(N, sigma, alpha)
% USWinMLW  Find x0 to set the width of the main lobe of an ultraspherical
% window to a given value.
% x0 = USWinMLW(N, sigma, alpha)
% x0    <-  Location in the ultraspherical polynomial in x corresponding to the
%           peak of the spectrum of the window.
% N     -> Number of window coefficients
% sigma  -> Main lobe width relative to the main lobe width of a rectangular
%           window. The main lobe of the spectrum of a window is defined by the
%           zero crossings surrounding the main lobe (centred at frequency 0).
%           The main lobe width is set to sigma*2/N (in normalized Hz).
% alpha  -> Window parameter
%           alpha=0 gives the Dolph-Chebyshev window (based on a Chebyshev
%             polynomial of the first kind)
%           alpha=1 gives the Saramäki window (based on a Chebyshev polynomial
%             of the second kind)

% $Id: USWinMLW.m,v 1.6 2025/03/19 14:50:05 pkabal Exp $

M = N - 1;

% For N = 1, the normalized window is [1]. The frequency response is constant,
%            independent of x0. There is no null in the spectrum. sigma is
%            undefined.
% For N = 2, the normalized window is [1; 1]. The frequency response is
%            2 cos(w/2), with a null at w=pi. sigma is 1, independent of x0.

if (N == 1)
  if (~isnan(sigma))
    warning('USWinMLW - main lobe width undefined for N = 1');
  end
  x0 = 1;
  return;
elseif (N == 2)
  if (sigma ~= N/2)
    warning('USWinMLW - incompatible main lobe width');
  end
  x0 = 1;
  return;
end

if (alpha == 0 || alpha == 1)
  x1 = EstZero(1, M, alpha);

else
  % Estimate the midway point between the first and second zeros
  x12 = 0.5 * (EstZero(1, M, alpha) + EstZero(2, M, alpha));

  % Search in the interval [x12, 1] for a zero crossing
  if (alpha < 0)
    xLim = [x12, 1.05];   % Zero can occur at x > 1
  else
    xLim = [x12, 1];
  end
  v = USPoly(xLim, M, alpha);
  if (sign(v(1)) == sign(v(2)))
    error('USWinMLW - Main lobe edge not found');
  end
  x1 = fzero(@USPoly, xLim, optimset('TolX', 1e-14), M, alpha);
  if (isnan(x1))
    error('USWinMLW: Failed to find the polynomial zero');
  end
end

% x1 = x0 cos(w1/2), solve for x0 from x1 and w1
w1 = sigma * 2*pi/N;  % sigma times first zero for a rectangular window
x0 = x1 / cos(w1/2);

end

% -----
function xr = EstZero(k, M, alpha)

% Estimate of the k'th zero of Pm(x)
% k = 1 for largest zero, k = 2 for the next largest zero, ...

% Interpolate/extrapolate the exact (inverse) roots of the Chebyshev polynomials
% (first and second kind)
xL0 = cos(pi * (k - 1/2) / M);    % Exact roots for alpha = 0
xL1 = cos(pi * k / (M+1));        % Exact roots for alpha = 1
xr = (xL0 * xL1) / (alpha * xL0 + (1 - alpha) * xL1);

end
