function x0 = USWinRdB(N, RdB, alpha)
% USWinRdB  Find x0 to set the peak of an ultraspherical window to be RdB above
% the level of the first sidelobe.
% x0 = USWinRdB(N, RdB, alpha)
% x0   <-  Location in the ultraspherical polynomial in x corresponding to the
%          peak of the spectrum of the window.
% N     -> Number of window coefficients
% RdB   -> Ratio of the peak to first sidelobe level in dB
% alpha -> Window parameter 
%          alpha=0 gives the Dolph-Chebyshev window (based on a Chebyshev
%            polynomial of the first kind)
%          alpha=1 gives the Saramäki window (based on a Chebyshev polynomial
%            of the second kind)

% $Id: USWinRdB.m,v 1.10 2025/03/19 14:50:49 pkabal Exp $

% Look for the first sidelobe in the interval just below x = 1. The search range
% is delimited by [xL, 1], where xL is an estimate of the zero location based on
% interpolating the zero locations of Chebyshev polynomials of the first and
% second kind.

dBVInv = @(y) (10^(y/20));

R = dBVInv(RdB);

% For N = 1, the normalized window is [1]. The frequency response is constant,
%            independent of x0. There is no null in the spectrum. R is
%            undefined.
% For N = 2, the normalized window is [1; 1]. The frequency response is
%            2 cos(w/2), with a null at w=pi. R is undefined.
if (N <= 2)
  if (~isnan(RdB))
    warning('USWinRdB - No sidelobe for N <= 2');
  end
  x0 = 1;
  return;
end

% Special case
if (alpha == 0)
  x0 = cosh(acosh(R) / (N-1));  % Returns x0 < 1 for R < 1
  return
end
  
M = N - 1;
xU = 1;
xL = EstZero(2, M, alpha);

% Find the sidelobe amplitude in the interval [xL, xU]
[xMin, SL] = fminbnd(@USP, xL, xU, optimset('TolX', 1e-14), M, alpha);
if (xMin == xL || xMin == xU || SL >= 0)
  error('USWinRdB: Failed to find the sidelobe level');
end

% Find the amplitude of the point x0
% Looking for C_M(x0) = R * |SL|.

% Look for the point on the polynomial equal to R*|SL|
xLim = [xMin, 1+10*(1-xMin)];  % [1-(1-xMin), 1+2*(1-xMin]
x0 = fzero(@USPsl, xLim, optimset('TolX', 1e-14), M, alpha, R*abs(SL));
if (isnan(x0))
  error('USWinRdB: Failed to find the peak level');
end

end

% -----
function xr = EstZero(k, M, alpha)

% Estimate of the k'th zero of Pm(x)
% k = 1 for largest zero, k = 2 for the next largest zero, ...

% Interpolate/extrapolate the exact (inverse) roots of the Chebyshev polynomials
% (first and second kind)
xL0 = cos(pi * (k - 1/2) / M);    % Exact roots for alpha = 0
xL1 = cos(pi * k / (M+1));        % Exact roots for alpha = 1
xr = (xL0 * xL1) / (alpha * xL0 + (1 - alpha) * xL1);

end

% -----
function v = USP(x, M, alpha)
% Ultraspherical polynomial, flipped in sign for alpha < 0

if (alpha < 0)
  v = -USPoly(x, M, alpha);
else
  v = USPoly(x, M, alpha);
end

end

% -----
function y = USPsl(x, M, alpha, RM)

if (alpha < 0)
  y = -USPoly(x, M, alpha) - RM;
else
  y = USPoly(x, M, alpha) - RM;
end

end
