function w = CTUSWint(tau, beta, alpha)
% CTUSWint  Continuous-time ultraspherical window
% w = CTUSWint(tau, beta, alpha)
% w    <-  Window sampled at times given by tau (column vector)
% tau   -> Sample times, where the non-zero extent of the window is -1 <= t <= 1
% beta  -> Window parameter (trades off main lobe width against sidelobe level)
% alpha -> Ultraspherical parameter, alpha >= 0
%
% The returned values are normalized by the value of the window at tau = 0. Note
% that for 0 <= alpha < 1, w(tau) for |tau| = 1 becomes infinite. This routine
% uses the Matlab modified bessel function.

% R. L. Streit, "A Two-Parameter Family of Weights for Nonrecursive Digital
%   Filters and Antennas", IEEE trans. Acoustics, Speech, Signal Proc.,
%   vol. ASSP-32, No. 1, pp. 108-118, Feb. 1984.

% $Id: CTUSWint.m,v 1.9 2025/01/29 15:53:42 pkabal Exp $

N = length(tau);
w = zeros(N, 1);    % Column vector
I = (abs(tau) <= 1);

w(I) = CTWin(tau(I), beta, alpha);

end

% ----- -----
function w = CTWin(tau, beta, alpha)
% Continuous-time ultraspherical window, normalized to w(0) = 1
% tau -> normalized time samples, non-zero part of window -1 <= tau <= +1.

d = (1-tau.^2);
nu = alpha-1;
if (beta == 0)
  w = d.^nu;
else
  w = sqrt(d).^nu .* besseli(nu, beta*sqrt(d)) / besseli(nu, beta);
end

I = isnan(w);
w(I) = Inf;

end
