function w = USChebyWinR2(N, x0, alpha, Norm)
% USWinR2  window design based on Chebyshev polynomials.
% w = USChebyWinR2(N, x0, alpha, Norm)
% w    <-  Symmetrical window coefficients (column vector), length N
% N     -> Number of window coefficients.
% x0    -> Location of the Chebyshev polynomial in x corresponding to the
%          peak of the spectrum of the window.
% N     -> Number of coefficients for the window
% alpha -> Window parameter
%          alpha=0 gives the Dolph-Chebyshev window (based on a Chebyshev
%            polynomial of the first kind)
%          alpha=1 gives the Saramäki window (based on a Chebyshev polynomial
%            of the second kind)
% Norm  -> Normalize flag (defaults to true if missing). If Norm is true, the
%          window coefficients are normalized by dividing the coefficients by
%          the element which has the largest absolute value - that element
%          becomes +1.
% This routine uses a double-step window recurrence relationship.

% $Id: USChebyWinR2.m,v 1.1 2025/04/14 13:44:08 pkabal Exp $

assert(alpha == 0 || alpha == 1);

if (nargin() < 4)
  Norm = true;
end

M = N-1;
Meven = (mod(M, 2) == 0);
x02 = x0^2;

if (Meven)
  mstart = 4;
  am = 2;
  % m = 0
  ce = 1;
  w = ce;

  if (M >= 2)
  % m = 1
    wm2 = w;
    if (alpha == 0)
      co = 1;
    else
      co = 2;
    end

  % m = 2;
    am = 2;
    w = x02/2 *co*[1; 2; 1] - [0; ce; 0];
  end

else
  mstart = 3;
  % m = -1
  w = [];

  % m = 1
  wm2 = 0.5*w;    % Scale by 2
  if (alpha == 0)
    am = 1;
  else
    am = 2;
  end
  w = (x0/2) * am * [1; 1];
end

% alpha == 1
%   a(m) = 2, m >= 1
% alpha == 0
%   a(m) = 1, m == 1
%        = 2, m >= 2
% b(m) = 1, m >= 1

% f(m)*x0^2 = a(m) * a(m-1) * x0^2,     m >= 2
% g(m) = b(m) + a(m)/a(m-2) * b(m-1),   m >= 3
% h(m) = a(m)/a(m-2) * b(m-1) * b(m-2), m >= 3

% f(m)*x0^2 = x0^2, m >= 3
% alpha == 1
%   g(m) = 2,   m >= 3
%   h(m) = 1,   m >= 3
% alpha == 0
%   g(m) = 3/2, m == 3
%        = 1,   m >= 4
%   h(m) = 1/2, m == 3
%        = 1,   m >= 4
% The value of h(m) is set to one for m >= 3. For the case of alpha = 0, and
% m == 3, wm4 is scaled by 1/2 to compensate.

% G = 2*f(m)*x0^2 - g(m)
% alpha == 1
%   G = 2*(x0^2 - 1), m >= 3
% alpha == 0
%   G = 2*x0^2 - 3/2, m == 3
%     = 2*(x0^2 - 1), m >= 4
gm = 1 + 2/am;
G = 2*x0^2 - gm;
for m = mstart:2:M
  wm4 = wm2;
  wm2 = w;
  w = x02*([wm2; 0; 0] + [0; 0; wm2]) + G*[0; wm2; 0] ...
                                      - [0; 0; wm4; 0; 0];
  G = 2*(x02 - 1);
end

% Normalize and make the largest value positive
if (Norm)
  [~, k] = max(abs(w));
  w = w / w(k);
elseif (alpha == 0 && M > 0)
  w = 2*w/M;      % Chebyshev I polynomial
end

end
