function p = USPolyCoef(M, alpha)
% UltraSphericalPolyCoef  Calculate the coefficients of an ultraspherical
% polynomial.
% p = USPolyCoef(M, alpha)
% p    <-  M+1 coefficients of a direct form polynomial in x, stored with the
%          coefficient of the highest power first. Every other term in p is
%          zero, i.e., p(n) = 0 when n is even.
% M     -> Polynomial order
% alpha -> Window parameter 
%          alpha=0 gives the Dolph-Chebyshev window (based on a Chebyshev
%            polynomial of the first kind)
%          alpha=1 gives the Saramäki window (based on a Chebyshev polynomial
%            of the second kind)
%
% For the ultraspherical polynomial C(x, M, alpha), this routine returns the
% coefficients of the polynomial expressed with powers of x.
%  C(x,M,alpha) = p(1) x^M + p(2) x^(M-1) + ... + p(M) x + p(M+1).
% Ignoring the possible computational savings due to the alternating zero
% values in p,
%   C(x,M,alpha) = polyval(p, x).
% Note that the coefficients p can get huge for large M, causing such a
% calculation to overflow.

% $Id: USPolyCoef.m,v 1.3 2025/02/16 17:45:32 pkabal Exp $

% pe contains even order terms, po contains odd order terms
if (alpha == 0)
  pe = 1;
  po = 1;
else
  pe = 1;         % m = 0 result
  po = 2*alpha;   % m = 1 result
end

Even = true;
for m = 2:M
  if (alpha == 0)
    if (Even)
      pe = 2*[po 0] - [0 pe];
    else
      po = 2*pe - [0 po];
    end
  else
    am = 2*(m+(alpha-1));
    bm = m+2*(alpha-1);
    if (Even)
      pe = ( am*[po 0] - bm*[0 pe] ) / m;
    else
      po = ( am*pe - bm*[0 po] ) / m;
    end
  end
  Even = ~Even;
end

p = zeros(1, M+1);
if (mod(M, 2) == 0)
  p(1:2:end) = pe;
else
  p(1:2:end) = po;
end

if (alpha == 0 && M > 0)
  p = (2/M) * p;
end

end
