function w = USWinR1(N, x0, alpha, Norm)
% USWinR1  Window design using an ultraspherical polynomial.
% w = USWinR1(N, x0, alpha, Norm)
% w    <-  Symmetrical window coefficients (column vector), length N
% N     -> Number of window coefficients.
% x0    -> Location of the ultraspherical polynomial in x corresponding to the
%          peak of the spectrum of the window.
% N     -> Number of coefficients for the window
% alpha -> Window parameter, special cases
%          alpha=0 gives the Dolph-Chebyshev window (based on a Chebyshev
%            polynomial of the first kind)
%          alpha=1 gives the Saramäki window (based on a Chebyshev polynomial
%            of the second kind)
% Norm  -> Normalize flag (defaults to true if missing). If Norm is true, the
%          window coefficients are normalized by dividing the coefficients by
%          the element which has the largest absolute value - that element
%          becomes +1.
% This routine uses the basic window recurrence relationship.

% $Id: USWinR1.m,v 1.2 2025/04/14 13:41:43 pkabal Exp $

if (alpha < 0 && fix(alpha) == alpha)
  error('USWin: alpha cannot be a negative integer');
end

if (nargin() < 4)
  Norm = true;
end

M = N-1;
w = 1;         % m = 0

if (M >= 1)
  % m = 1
  x0b2 = x0/2;
  wm1 = w;
  if (alpha == 0)
    w = x0b2*[1; 1];
    nu2 = 0;
  else
    w = 2*alpha*x0b2*[1; 1];
    nu2 = 2*(alpha-1);
  end
end

for m = 2:M
  am = 2 + nu2/m;
  bm = 1 + nu2/m;
  wm2 = wm1;
  wm1 = w;
  w = am*x0b2*([wm1; 0] + [0; wm1]) - bm*[0; wm2; 0];
end

% Normalize and make the largest value positive
if (Norm)
  [~, k] = max(abs(w));
  w = w / w(k);
elseif (alpha == 0 && M > 0)
  w = 2*w/M;      % Chebyshev I polynomial
end

end
