function w = USWinR1A(N, x0, alpha, Norm)
% USWinR1A  Window design using an ultraspherical polynomial.
% w = USWinR1A(N, x0, alpha, Norm)
% w    <-  Symmetrical window coefficients (column vector), length N
% N     -> Number of window coefficients.
% x0    -> Location of the ultraspherical polynomial in x corresponding to the
%          peak of the spectrum of the window.
% N     -> Number of coefficients for the window
% alpha -> Window parameter, special cases
%          alpha=0 gives the Dolph-Chebyshev window (based on a Chebyshev
%            polynomial of the first kind)
%          alpha=1 gives the Saramäki window (based on a Chebyshev polynomial
%            of the second kind)
% Norm  -> Normalize flag (defaults to true if missing). If Norm is true, the
%          window coefficients are normalized by dividing the coefficients by
%          the element which has the largest absolute value - that element
%          becomes +1.
% This routine uses an alternate window recurrence relationship.

% $Id: USWinR1A.m,v 1.3 2025/04/14 13:42:11 pkabal Exp $

if (alpha < 0 && fix(alpha) == alpha)
  error('USWin: alpha cannot be a negative integer');
end

if (nargin() < 4)
  Norm = true;
end

M = N-1;
meven = true;
ce = 1;         % m = 0

if (M >= 1)
  meven = ~meven;
  if (alpha == 0)
    co = 1;
    nu2 = 0;
  else
    co = 2*alpha;  % m = 1, scaled by x
    nu2 = 2*(alpha-1);
  end
end

x02 = (x0/2)^2;
for m = 2:M
  meven = ~meven;

  am = 2 + nu2/m;
  bm = 1 + nu2/m;
  if (meven)
    % Group calculations so that exact symmetry is maintained
    ce = am*x02*( ([co; 0; 0] + [0; 0; co]) + 2*[0; co; 0] ) - bm*[0; ce; 0];
  else
    co = am*ce - bm*[0; co; 0];
  end
end

if (~meven)
  w = (x0/2)*([co; 0] + [0; co]);
else
  w = ce;
end

% Normalize and make the largest value positive
if (Norm)
  [~, k] = max(abs(w));
  w = w / w(k);
elseif (alpha == 0 && M > 0)
  w = 2*w/M;      % Chebyshev I polynomial
end

end
