function w = USWinR2(N, x0, alpha, Norm)
% USWinR2  window design using an ultraspherical polynomial.
% w = USWinR2(N, x0, alpha, Norm)
% w    <-  Symmetrical window coefficients (column vector), length N
% N     -> Number of window coefficients.
% x0    -> Location of the ultraspherical polynomial in x corresponding to the
%          peak of the spectrum of the window.
% N     -> Number of coefficients for the window
% alpha -> Window parameter, special cases
%          alpha=0 gives the Dolph-Chebyshev window (based on a Chebyshev
%            polynomial of the first kind)
%          alpha=1 gives the Saramäki window (based on a Chebyshev polynomial
%            of the second kind)
% Norm  -> Normalize flag (defaults to true if missing). If Norm is true, the
%          window coefficients are normalized by dividing the coefficients by
%          the element which has the largest absolute value - that element
%          becomes +1.
% This routine uses a double-step window recurrence relationship.

% $Id: USWinR2.m,v 1.5 2025/04/14 13:43:09 pkabal Exp $

if (alpha < 0 && fix(alpha) == alpha)
  error('USWin: alpha cannot be a negative integer');
end

if (nargin() < 4)
  Norm = true;
end

if (alpha == 0)
  nu2 = 0;
else
  nu2 = 2*(alpha-1);
end

M = N-1;
Meven = (mod(M, 2) == 0);
x02 = (x0/2)^2;

if (Meven)
  mstart = 4;
  % m = 0
  ce = 1;
  w = ce;

  if (M >= 2)
  % m = 1
    wm2 = w;
    if (alpha == 0)
      co = 1;
    else
      co = 2*alpha;
    end

    m = 2;
    am = 2 + nu2/m;
    bm = 1 + nu2/m;
    w = am* (x0/2)^2 *co*[1; 2; 1] - bm*[0; ce; 0];
  end

else
  mstart = 3;
  % m = -1
  w = [];

  % m = 1
  wm2 = w;
  if (alpha == 0)
    am = 1;
  else
    am = 2*alpha;
  end
  bm = 1 + nu2;
  w = (x0/2) * am * [1; 1];
 
end


for m = mstart:2:M
  wm4 = wm2;
  wm2 = w;

  am2 = am;
  am1 = 2 + nu2/(m-1);
  am  = 2 + nu2/m;

  bm2 = bm;
  bm1 = 1 + nu2/(m-1);
  bm  = 1 + nu2/m;
  
  fm = am * am1;
  gm = bm + (am * bm1)/am2;
  hm = ((am * bm1)/am2) * bm2;

  w = fm*x02*([wm2; 0; 0] + [0; 0; wm2]) + (2*fm*x02-gm)*[0; wm2; 0] ...
      - hm*[0; 0; wm4; 0; 0];
end

% Normalize and make the largest value positive
if (Norm)
  [~, k] = max(abs(w));
  w = w / w(k);
elseif (alpha == 0 && M > 0)
  w = 2*w/M;      % Chebyshev I polynomial
end

end
