function h = USWinSeries(N, x0, alpha, Norm)
% USWinSeries  Window based on an ultraspherical polynomial - series calculation
% h = USWinSeries(N, x0, alpha)
% h    <-  Symmetrical window coefficients (column vector), length N
%          which has the largest absolute value - that element becomes +1.
% N     -> Number of window coefficients
% x0    -> Location of the ultraspherical polynomial in x corresponding to the
%          peak of the spectrum of the window.
% alpha -> Window parameter
%          alpha=0 gives the Dolph-Chebyshev window (based on a Chebyshev
%            polynomial of the first kind)
%          alpha=1 gives the Saramäki window (based on a Chebyshev polynomial of
%            the second kind)
% Norm  -> Normalize flag (defaults to true if missing). If Norm is true, the
%          window coefficients are normalized by dividing the coefficients by
%          the element which has the largest absolute value - that element
%          becomes +1.
%
% R. L. Streit, "A Two-Parameter Family of Weights for Nonrecursive Digital
%   Filters and Antennas", IEEE Trans. Acoustics, Speech, Signal Processing,
%   vol. ASSP-32, no. 1, pp, 108-118, Feb. 1994.
% S. W. A. Bergen and A. Antoniou, "Design of Ultraspherical Window Functions
%   with Prescribed Spectral Characteristics", EURASIP J. Applied Signal
%   Processing, pp. 2053-2065, vol. 2004, no. 13, October 2004.

% $Id: USWinSeries.m,v 1.14 2025/03/31 15:56:21 pkabal Exp $

% Streit
%   b(n,k)(zmu) = 2 mu zmu^n / (n-k) c(mu+n-k-1,n-k-1)
%                    sum(m=0:k) c(mu+k-1, k-m) c(n-k,m) A^m
%     n = M = N-1
%     zmu = x0;
%     mu = alpha
%     A = 1 - 1/x0^2
%     k runs from 0 to floor(M/2)
%   b(M,k)(x0) = 2 alpha x0^M / (M-k) c(alpha+M-k-1, M-k-1)
%                    sum(m = 0:n) c(alpha+k-1, k-m) c(M-k,m) A^m
% Switching to the form in Bergen, omitting the scaling factor 2,
%   h(n) = A / (M-n) c(alpha+M-n-1), M-n-1)
%                    sum(m = 0:n) c(alpha+n-1, n-m) c(M-n,m) D0^m
% where A = alpha x0^M, for alpha ~= 0, and A = x0^M for alpha = 0
%       D0 = 1 - 1/x0^2
%       n replaces k as the time index

GenBinCoef = @(a, k)(exp(gammaln(a+1) - gammaln(k+1) - gammaln(a-k+1)));

if (nargin() < 4)
  Norm = true;
end

M = N-1;
if (alpha == 0)
  A = x0^M;
else
  A = alpha * x0^M;
end

nh = floor(M/2);    % N even, h[n] for n = 0:N/2-1
                    % N odd,  h[n] for n = 0:(N-1)/2
D0 = 1 - 1/x0^2;

% Pre-calculate B^m, check for an underflow
Bv = zeros(nh+1, 1);
Bv(0+1) = 1;
mmx = nh;
for m = 1:nh
  D0m = D0^m;
  if (D0m == 0)
    mmx = m-1;
    break;
  end
  Bv(m+1) = D0m;
end

h = NaN(N, 1);
for n = 0:nh
  s = 0;
  for m = 0:min(n, mmx)
    cm = BinomCoef(alpha+n-1, n-m) * BinomCoef(M-n, m);
    s = s + cm * Bv(m+1);
  end
 
  Cn = (A/(M-n)) * GenBinCoef(alpha+M-n-1, M-n-1);
  h(n+1) = Cn * s;
  h(M-n+1) = h(n+1);
end

% Normalize and make the largest value positive
if (Norm)
  [~, k] = max(abs(h));
  h = h / h(k);
end

if (any(isnan(h)))
  warning('USWinSeries: NaN coefficients');
end

end

% ----- -----
function BC = BinomCoef(a, k)
% Generalized binomial coefficient

% From Streit (17)
%  C(a,k) (a choose k), where a is a real value
%  C(a,k) = 1, for k = 0
%  C(a,k) = (a-k+1)/k C(a,k-1), for k >= 1

% The generalized binomial coefficient satisfies
%             Inf
% (1 + x)^a = SUM C(a,k) x^k
%             k=0

BC = 1;
while (k > 0)
  BC = BC * ((a - k + 1) / k);
  k = k - 1;
end

end

