function CheckDiaryFile = StartDiaryFile(varargin)
% CheckDiaryFile = StartDiaryFile(LogFile, RefFile, SysInfo)
% CheckDiaryFile = StartDiaryFile(SysInfo)
% CheckDiaryFile = StartDiaryFile()
% ChekcDiaryFile(SearchStr, ReplaceStr)
% CheckDiaryFile()
%
% StartDiaryFile:
% CheckDiaryFile <-  Pointer to function to compare the log file to the
%                    reference file
% LogFile         -> Diary file containing the output text (the default name
%                    is the module name with a ".log" extension)
% RefFile         -> Reference file containing past diary file output, default
%                    name determined from the log file name changed to have a
%                    ".ref" extension.
% SysInfo         -> Logical flag, if set, the diary file will have a header
%                    containing the Matlab version and the hostname of the
%                    computer. The default is off. This argument can be the last
%                    argument if either or both of the LogFile and RefFile
%                    arguments are missing.
%
% CheckDiaryFile:
% SearchStr   -> Search string or cell array of search strings
% ReplaceStr  -> Replace string or cell array of replace strings
%
% To check the correct functioning of a Matlab function, a test routine can be
% written to exercise the Matlab function with different test cases. The output
% of the test cases can be written to the screen and simultaneously to a diary
% file. StartDiarFile/CheckDiaryFile will determine whether changes to the
% Matlab function result in changes to the output of the test routine as stored
% in the diary file.
%
% Using StartDiaryFile, output from the test function is routed to the diary
% file (here called the log file). Using the companion routine CheckDiaryFile,
% the log file is compared with a reference log file. The reference file
% contains the expected output of the test script. StartDiaryFile supplies
% defaults for LogFile and RefFile based on the name of the routine calling this
% routine. If the calling function is testfn.m, the defaults are 'testfn.log'
% and 'testfn.ref'.
%
% CheckDiaryFile allows for string replacement in the log file. An example of
% this is to replace a path string with a canonical string.
%   CheckDiaryFile(pwd, '--path--');
%
% StartDiaryFile:
% 1. Close any open diary files.
% 2. Start the diary file. The Matlab diary is sent to the log file (if the log
%    file name is not empty).
% 3. Save the current output format parameters. Set the output format to 'short'
%    and 'compact'.
% 4. Return a pointer to the routine CheckDiaryFile. The saved format parameters
%    and the file names for the log and reference files are available to
%    CheckDiaryFile.
%
% CheckDiaryFile:
% 1. Close the diary file.
% 2. Reset the output formatting to the settings prior to calling SetDiaryFile.
% 3(a) If the log file name is empty, no further action is taken.
%  (b) String substitution is performed on the log file.
%  (c) If the log file name differs from the reference file name, the contents
%      of these files are compared using the system file comparison tool.
%  (d) If the file contents are identical, the log file is deleted.
%
% Notes:
% - For a first run with no reference file, the log file will be created. That
%   log file can be renamed to become the reference file for subsequent runs.
% - When Matlab displays the contents of a matrix (disp command for instance),
%   the number of elements displayed per line depends on the width of the Matlab
%   command window. This means that a comparison of the log and reference files
%   may show differences when the numerical values are unchanged. The workaround
%   is to print matrices using formatted I/O.
%
% StartDiaryFile should be called at the beginning of the test script and
% CheckDiaryFile should be called at the end of the test script.
%   CheckDiaryFile = StartDiaryFile(LogFile, RefFile)
%   ...
%   CheckDiaryFile();
%
% Note that if the script fails before CheckDiaryFile turns diary off, the log
% file can be locked, which may require restarting Matlab to remove the lock.

% $Id: StartDiaryFile.m,v 1.20 2024/12/11 19:12:38 pkabal Exp $

% Process the arguments
[LogFile, RefFile, SysInfo] = DecArgs(varargin{:});

% Get calling module name
if ((isempty(LogFile) || isempty(RefFile)) && size(dbstack(), 1) > 1)
  ST = dbstack(1);            % Skip one entry (this file)
  Cmodule = ST(1).name;
else
  Cmodule = 'CommandWindow';  % Called from the Command window
end

if (isempty(LogFile))
  LogFile = [Cmodule, '.log'];
end
if (isempty(RefFile))
  [LogFilePath, LogFileName] = fileparts(LogFile);
  RefFile = fullfile(LogFilePath, [LogFileName, '.ref']);
end

diary('off');
if (~isempty(LogFile))
  if (~isempty(ls(LogFile)))
    % May need to pause here to let diary('off') take effect
    delete(LogFile);  % If it exists, we don't want to append to it
  end
  diary(LogFile);
end

% Before R2018a:
%   NumericFormat = get(0, 'Format');
%   DisplayLineSpacing = get(0, 'FormatSpacing');
% R2018a
s = settings;
NumericFormat = s.matlab.commandwindow.NumericFormat.ActiveValue;
DisplayLineSpacing = s.matlab.commandwindow.DisplayLineSpacing.ActiveValue;
% R2021a
%  fmt = format();
%  NumericFormat = fmt.NumericFormat;
%  DisplayLineSpacing = fmt.LineSpacing;
format('short');
format('compact');

% Header (add a blank line)
if (SysInfo)
  [~, hostname] = system('hostname');   % May have trailing CR-LF
  fprintf('--- Matlab %s on %s ---\n\n', ...
          version('-release'), deblank(hostname));
end

CheckDiaryFile = @Check_DiaryFile;

% ----- begin nested function
function Check_DiaryFile(SearchStr, ReplaceStr)
% This function compares a log file with a reference file. It is the companion
% function to SetDiaryFile. The parameters are inherited from StartDiaryFile.
%
% 1. Close the diary.
% 2. Reset the output formatting to the settings prior to calling SetDiaryFile.
% 3(a) If the log file name is empty, no further action is taken.
%  (b) If the log file name differs from the reference file name, the contents
%      of these files are compared using the system file comparison tool.
%  (c) If the files have identical content, the log file is deleted.

% Test for non-empty character vector
isCharVector = @(s) (ischar(s) && isrow(s)); 
% Test for possibly-empty character vector
isCharVectZ = @(s) (isempty(s) || (ischar(s) && isrow(s)));

diary('off');

% Restore formatting
format(NumericFormat);
format(DisplayLineSpacing);

% Replace strings
if (nargin() == 2 && isCharVector(SearchStr) && isCharVectZ(ReplaceStr))
  Fstrrep(LogFile, SearchStr, ReplaceStr);
end

if (~isempty(ls(RefFile)) && ~strcmp(LogFile, RefFile))
  fprintf('\n***** ***** ***** *****\n');

  % System file compare
  if (ispc())
    Status = system(['FC ', LogFile, ' ', RefFile]);
  else
    Status = system(['diff ', LogFile, ' ', RefFile]);
  end

  % Delete the log file if the files are identical
  if (Status == 0)
    delete(LogFile);
  end
end

end
% ----- end nested function

end

% ----- -----
function [LogFile, RefFile, SysInfo] = DecArgs(varargin)

% Test for non-empty character vector
isCharVector = @(s) (ischar(s) && isrow(s));
% Test for non-empty logical scalar
isLogicalScalar = @(x) (islogical(x) && isscalar(x));

LogFile = '';
RefFile = '';
SysInfo = false;

if (~isempty(varargin) && isCharVector(varargin{1}))
  LogFile = varargin{1};
  varargin(1) = [];
  if (~isempty(varargin) && isCharVector(varargin{1}))
    RefFile = varargin{1};
    varargin(1) = [];
  end
end

if (~isempty(varargin) && isLogicalScalar(varargin{1}))
  SysInfo = varargin{1};
  varargin(1) = [];
end

if (~isempty(varargin))
  error('StartDiaryFile: Too many input arguments');
end

end

% ----- -----
function Fstrrep(FName, SearchStr, ReplaceStr)
% Read a text file, replace strings, overwrite the text file

% Read the input text file (as binary to preserve CR/LF)
FID = fopen(FName, 'rb');
StrIN = (fread(FID, 'char=>char'))';    % Convert to row vector
fclose(FID);

StrOUT = strrep(StrIN, SearchStr, ReplaceStr);
if (~isequal(StrOUT, StrIN))
  FID = fopen(FName, 'wb');   % Overwrite the file
  fwrite(FID, StrOUT);
  fclose(FID);
end

end
