function tUSWin()

% Test ultraspherical windows
% - Ultraspherical windows are calculated as the inverse of the sampled
%   spectrum, using a window recurrence, and using a series expansion.
%
% For specific windows, other versions of the window are also evaluated
% - Chebyshev (first kind): Compare with MATLAB Dolph-Chebyshev window and
%   Saramäki construction (only for odd lengths)
% - Chebyshev (second kind): Saramäki construction
% - Legendre window
% - Large alpha
% - Negative alpha

% $Id: tUSWin.m,v 1.17 2025/03/26 18:21:10 pkabal Exp $

SysInfo = true;
CheckDiaryFile = StartDiaryFile(SysInfo);
SavePath = addpath('..', '../extras');

RdB = 30;

% Dolph-Chebyshev window
alpha = 0;
PrHead('Chebyshev (first kind) windows', RdB, alpha)
for N = [20, 21, 101]
  TestUSWin(N, RdB, alpha);
end

% Saramäki window
alpha = 1;
PrHead('+Chebyshev (second kind) windows', RdB, alpha);
for N = [20, 21]
  TestUSWin(N, RdB, alpha);
end

% Legendre window
alpha = 0.5;
PrHead('+Legendre windows', RdB, alpha);
for N = [20, 21]
  TestUSWin(N, RdB, alpha);
end

% Negative alpha, larger positive alpha
for alpha = [-0.5, 2]
  PrHead('+US windows', RdB, alpha);
  for N = [20, 21]
    TestUSWin(N, RdB, alpha);
  end
end

path(SavePath);
CheckDiaryFile();

end

% -----
function PrHead(string, RdB, alpha)

if (string(1) == '+')
  fprintf('\n');
  string(1) = '';
end
fprintf('%s: alpha = %g, RdB = %g\n', string, alpha, RdB);

end

% -----
function hD = TestUSWin(N, RdB, alpha)

fprintf('N = %d\n', N);

x0 = USWinRdB(N, RdB, alpha);

% Check FindUSWinRdB
assert(abs(RdB - FindUSWinRdB(N, x0, alpha)) < 1e-8);

sigma = FindUSWinMLW(N, x0, alpha);
MLW = sigma * (2/N);
fprintf('  Main Lobe Width: %.3g/N\n', N*MLW);

% Generate the window
hD = USWin(N, x0, alpha);

hR1 = USWinR1(N, x0, alpha);
fprintf('  IDFT/R1,     max error: %.3g\n', max(abs(hD - hR1)));
hR1A = USWinR1A(N, x0, alpha);
fprintf('  IDFT/R1A,    max error: %.3g\n', max(abs(hD - hR1A)));
hR2 = USWinR2(N, x0, alpha);
fprintf('  IDFT/R2,     max error: %.3g\n', max(abs(hD - hR2)));
hS = USWinSeries(N, x0, alpha);
fprintf('  IDFT/series, max error: %.3g\n', max(abs(hD - hS)));

% Dolph-Chebyshev window
if (alpha == 0)
  CheckCheby1(hD, x0);
end

end

% ----- -----
function CheckCheby1(h, x0)
% h  - window generated by USWin
% x0 - window parameter

dBVInv = @(x)(10.^(x/20));

alpha = 0;
N = length(h);

% Check DT Dolph-Chebyshev formula for R <-> x0
RdB = FindUSWinRdB(N, x0, alpha);
R = dBVInv(RdB);
assert(abs(x0 - cosh(acosh(R)/(N-1))) < 1e-12)

% Check DT Dolph-Chebyshev formula for x1 => wz => MLW
sigma = FindUSWinMLW(N, x0, alpha);
MLW = sigma * (2/N);
x1 = cos(pi/(2*(N-1)));
wz = 2 * acos(x1/x0);   % x1 = x0*cos(wz/2) => wz = 2*acos(x1/x0)
assert(abs(MLW - 2*wz/(2*pi)) < 1e-14);

% MATLAB Dolph-Chebyshev
hDC = chebwin(N, RdB);
fprintf('  Matlab chebwin: RMSErr = %.3g\n', RMSErrOptGain(h, hDC));

end
