function tUSWinCalc()
% Compare ultraspherical window calculations

% $Id: tUSWinCalc.m,v 1.15 2025/03/31 15:55:40 pkabal Exp $

SysInfo = true;
CheckDiaryFile = StartDiaryFile(SysInfo);

SavePath = addpath('..', '../extras');

% Test cases
Nw = [23, 24, 239, 240, 1023, 1024];
RdB = 60;
alphaV = [0, 0.5, 1, 1.5, 2];

for alpha = alphaV
  fprintf('----- -----\n')
  for N = Nw
    % Find x0 which gives RdB of sidelobe attenuation
    x0 = USWinRdB(N, RdB, alpha);

    RdB = FindUSWinRdB(N, x0, alpha);
    sigma = FindUSWinMLW(N, x0, alpha);
    fprintf('N = %d, alpha = %g, x0 = %.7g, R = %.3g dB, sigma = %.3g\n', ...
            N, alpha, x0, RdB, sigma);
    RunWinTest(N, x0, alpha);
    fprintf('\n');
  end
end

path(SavePath);
CheckDiaryFile();

end

% ----- -----
function RunWinTest(N, x0, alpha)

% Windows
hD = USWin(N, x0, alpha);
hR1A  = USWinR1A(N, x0, alpha);
hS = USWinSeries(N, x0, alpha);

% Window comparisons
fprintf('Reference window: USWin IDFT\n');
Cmph('Win: recurrence ', hD, hR1A);
Cmph('Win: series     ', hD, hS);
[nterm, nmax] = SeriesMxTerms(N, x0);
fprintf('  Win: series, No. terms %d -> %d\n', nmax, nterm);

% Alternate IDFT implementations
if (mod(N, 2) == 0)
  heA = RealNEvenIDFTA(N, x0, alpha);
  RedPoly = true;
  heB = RealNEvenIDFTA(N, x0, alpha, RedPoly);
  Cmph('Win: N Even IDFT A', hD, heA);
  Cmph('Win: N Even IDFT B', hD, heB);
end

% Compare the DFT of the window with the spectrum
f = (0:N-1) / N;
w = 2*pi*f;

HRU = USPolySpec(w, N, x0, alpha);
fprintf('\nReference spectrum: USPoly\n');

HRUs = USPolyCoefSpec(w, N, x0, alpha);
if (any(isnan(HRUs)))
  fprintf('  *** USPolyCoef spectrum NaN\n');
else
  MaxPolyCoefErr = max(abs(HRU - HRUs));
  if (any(isnan(HRUs)) || MaxPolyCoefErr > 1e-10)
    fprintf('  *** Inaccurate USPolyCoef spectrum, %.3g\n', MaxPolyCoefErr); 
  end
end

HD = WinSpec(hD);
HR1A =  WinSpec(hR1A);
HS = WinSpec(hS);

fprintf('  Spec: IDFT       %.3g\n', max(abs(HRU-HD)));
fprintf('  Spec: recurrence %.3g\n', max(abs(HRU-HR1A)));
fprintf('  Spec: series     %.3g\n', max(abs(HRU-HS)));

if (alpha == 0 || alpha == 1)
  fprintf('Reference spectrum: Chebyshev polynomial\n');
  HRC = ChebyPolySpec(w, N, x0, alpha);
  fprintf('  Spec: IDFT       %.3g\n', max(abs(HRC-HD)));
  fprintf('  Spec: recurrence %.3g\n', max(abs(HRC-HR1A)));
  fprintf('  Spec: series     %.3g\n', max(abs(HRC-HS)));
end

end

% ----- -----
function H = USPolySpec(w, N, x0, alpha)
% Spectrum calculated using USPoly

% Sample the US polynomial
x = x0 * cos(w/2);
H = USPoly(x, N-1, alpha);

% Normalize
H = H / H(0+1);

end

% ----- -----
function H = USPolyCoefSpec(w, N, x0, alpha)
% Spectrum calculated using USPolyCoef

% Generate the US polynomial coefficients
p = USPolyCoef(N-1, alpha);

% Evaluate the polynomial
x = x0 * cos(w/2);
H = polyval(p, x);

% Normalize
H = H / H(0+1);

end

% ----- -----
function HR = ChebyPolySpec(w, N, x0, alpha)
% Spectrum calculated using Chebyshev polynomials

x = x0 * cos(w/2);
switch alpha
  case 0
    HR = Cheby1Poly(x, N-1);
  case 1
    HR = Cheby2Poly(x, N-1);
  otherwise
    error('Invalid alpha value');
end

% Normalize
HR = HR / HR(0+1);

end

% ----- -----
function H = WinSpec(h)

N = length(h);
f = (0:N-1) / N;
w = 2*pi*f;
H = zerophase(h, 1, w);

% Normalize
H = H / H(0+1);

end

% ----- -----
function Cmph(Banner, hr, h)

if (size(hr) ~= size(h))
  error('Incompatible vectors');
end

% Compare with reference window
MaxDiff = max(abs(h-hr));
fprintf('  %s %.3g\n', Banner, MaxDiff);
if (MaxDiff > 1e-6)
  warning('*** Large window differences')
end

% Check for anomalies
serr = max(abs(h - h(end:-1:1)));
if (serr)
 fprintf('  *** Symmetry error: %g ***\n', serr);
end

if (any(h <= 0))
  fprintf('  *** Window nonpositive ***\n');
end

if (any(h(2:end-1) > 1))
  max(h)
  fprintf('  *** Window > 1 ***\n');
end

% Check for local extrema
% Find the local maxima and minima - extrema have slopes of different signs on
% either side - IExt corresponds to points 2, ..., N-1
hdiff = diff(h);                                      % Length N-1
IExt = find(hdiff(1:end-1) .* hdiff(2:end) < 0) + 1;  % 1-based indices

N = length(h);
Next = length(IExt);
if (Next > 8)
  fprintf('  *** Found %d window extrema ***\n', Next);
else
  if ((mod(N, 2) ~= 0 && Next > 1) || (mod(N, 2) == 0 && Next > 0))
    fprintf('  *** Window extrema: [ ');
    for i = 1:Next
      fprintf('%d ', IExt(i));
    end
    fprintf('] ***\n');
  end
end

end

% ----- -----
function [nterm, nmax] = SeriesMxTerms(N, x0)

M = N-1;
nh = floor(M/2);
B = 1 - 1/x0^2;

% Find underflow in B^m
mmax = nh;    % Index of m (zero-based) which still has a non-zero B^m
for m = 1:nh
  if (B^m == 0)
    mmax = m-1;
    break;
  end
end

% No. terms is mmax+1
nterm = mmax+1;
nmax = nh+1;

end

% ----- -----
function h = RealNEvenIDFTA(N, x0, alpha, RedPoly)

if (mod(N, 2) ~= 0)
  error('N must be even');
end

if (nargin() < 4)
  RedPoly = false;
end

% Sample the polynomial at (No-1)/2 values, where No = N-1
No = N - 1;
fo = (0:(No-1)/2) / No;
wo = 2*pi*fo;
xo = x0 * cos(wo/2);

% Sample the scaled response at (No-1)/2+1 values
if (RedPoly)
  HRo = USPoly(xo, N-1, alpha, RedPoly);
else
  HRo = USPoly(xo, N-1, alpha) ./ xo;
end
ho = IDFTO(HRo);
h = 0.5*conv([1 1], ho);

% Normalize and make the largest value positive
[~, k] = max(abs(h));
h = h / h(k);

end

% ----- -----
function h = IDFTO(B)
% Inverse DFT for a linear-phase frequency response
% h <-  Output time samples, h[n] for 0 <= n <= N-1 with h[n] = h[N-1-n].
% B  -> Input zero-phase spectral samples, B[k] for 0 <= k <= (N-1)/2, where
%       N is odd.

NB = length(B);
N = 2*NB - 1;
nm = (N-1)/2;

h = NaN(N, 1);
kh = 1:NB-1;
for n = 0:nm-1
  h(n+1) = ( B(0+1) + 2 * B(kh+1) * cos(2*pi*(n-nm)*kh/N)' )/N;
  h(N-1-n+1) = h(n+1);
end
h(nm+1) = ( B(0+1) + 2 * sum(B(kh+1)) )/N;

end
